package com.demo.springboot_reactive_mongo.controller;


import com.demo.springboot_reactive_mongo.dto.ProductDto;
import com.demo.springboot_reactive_mongo.service.ProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/products")
public class ProductController {

    @Autowired
    private ProductService service;

    @GetMapping("/product/allProducts")
    public Flux<ProductDto> getAllProducts(){
        return service.getAllProducts();
    }

    @GetMapping("/product/{id}")
    public Mono<ProductDto> getProduct(@PathVariable String id){
        return service.getProduct(id);
    }

    @GetMapping("/product/price/{id}")
    public Mono<Double> getProductPrice(@PathVariable String id){
        return service.getProductPrice(id);
    }

    @GetMapping("/product/product-range")
    public Flux<ProductDto> getProductBetweenRange(@RequestParam("min") double min,@RequestParam double max){
        return service.getProductInRange(min, max);
    }

    @PostMapping("/product/addProduct")
    public Mono<ProductDto> saveProduct(@RequestBody Mono<ProductDto> productDtoMono){

        return productDtoMono
                .flatMap(productDto -> service.saveProduct(Mono.just(productDto)));


    }

    @PostMapping("/update/{id}")
    public Mono<ProductDto> updateProduct(@RequestBody Mono<ProductDto> productDtoMono, @PathVariable String id){
        return service.updateProduct(productDtoMono,id);

    }

    @DeleteMapping("/delete/{id}")
    public Mono<Void> deleteProduct(@PathVariable String id){
        return service.deleteProduct(id);
    }
}
