package com.demo.springboot_reactive_mongo.service;

import com.demo.springboot_reactive_mongo.dto.ProductDto;
import com.demo.springboot_reactive_mongo.entity.Product;
import com.demo.springboot_reactive_mongo.repository.ProductRepository;
import com.demo.springboot_reactive_mongo.utils.AppUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Range;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestBody;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class ProductService {

    @Autowired
    private ProductRepository repository;



    public Flux<ProductDto> getAllProducts(){
        return repository.findAll().map(AppUtils::entityToDto);
    }

    public Mono<ProductDto> getProduct(String id){
        return repository.findById(id).map(AppUtils::entityToDto);
    }

    public Mono<Double> getProductPrice(String id){
        return repository.findById(id).map(AppUtils::entityToDto).map(dto -> dto.getPrice());
    }

    public Flux<ProductDto> getProductInRange(double min,double max){
        return repository.findByPriceBetween(Range.closed(min,max));
    }

    public Mono<ProductDto> saveProduct(Mono<ProductDto> productDtoMono){
      return  productDtoMono.map(AppUtils::dtoToEntity)
                .flatMap(repository::insert) //flatmap because it is using Mono of mono list
                .map(AppUtils::entityToDto);

    }

    public Mono<ProductDto> updateProduct(Mono<ProductDto> productDtoMono,String id){
        return  repository.findById(id)
                .flatMap(p-> productDtoMono.map(AppUtils::dtoToEntity)
                .doOnNext(e -> e.setId(id)))
                .flatMap(repository::save)
                .map(AppUtils::entityToDto);
    }

    public Mono<Void> deleteProduct(String id){
        return repository.deleteById(id);
    }
}
